#if USING_USTL
    #include <ustl.h>
#else
    #include <iostream>
    #include <fstream>
    #include <map>
#endif

// define short name for map type used in this program
typedef std::map< std::string, std::string, std::less< std::string > > MapDict;
typedef void (*csv_cb1)(void*, size_t, void*);
typedef void (*csv_cb2)(int, void*);

MapDict _translations_;
static bool _translations_is_dirty = false;
int trcount = 0;

int menu_menu_bar_i18n_done = 0;
int menu_pedidos_popup_i18n_done = 0;

static const char *translations_file_name = "translations.csv";

#include "csv.h"

class CSV_Parser {
    protected:
        struct csv_parser p;
        int fld_idx;
        std::string tr_key, tr_value;
        MapDict *map;

    static void field_ready_cb(char *field, size_t size, CSV_Parser *p){
        //std::cout << field << " : " << size << std::endl;
        switch(p->fld_idx){
            case 1: p->tr_key.assign(field, size);
                break;

            case 2: p->tr_value.assign(field, size);
                break;
        }
        p->fld_idx++;
    };

	static void record_ready_cb(int chr, CSV_Parser *p){
        p->fld_idx = 1;
        //p->map->insert( MapDict::value_type( p->tr_key, p->tr_value ) );
        MapDict &m = *p->map;
        //std::cout << p->tr_key << " : " << p->tr_value << "\n";
        m[p->tr_key] = p->tr_value;
    };

    public:
    CSV_Parser() {
        csv_init(&p, 0);
        fld_idx = 1;

    };
    ~CSV_Parser(){
        csv_free(&p);
    };

    MapDict*  parse(std::string file_name, MapDict &map_in) {
        std::ifstream input(file_name.c_str(), std::ios::in | std::ios::binary);
        if(input.is_open()){
            map = &map_in;
            char content[1024];
            int count;
            while(!input.eof()){
                input.read(content, 1024);
                count = input.gcount();
                //std::cout << count << std::endl;
                csv_parse(&p, content, count,
                              (csv_cb1)&field_ready_cb,
                              (csv_cb2)&record_ready_cb,
                              this);

/*
                if (csv_parse(&p, content.c_str(), count,
                              &field_ready_cb,
                              cb2,
                              outfile) != i)
*/
            }
        }
        return map;
    };
};

int load_translations(){
    //std::cout <<  "load_translations\n" << trcount << std::endl;
    CSV_Parser csv;
    csv.parse(translations_file_name, _translations_);
    // Iterate over the map and print out all key/value pairs.
    // Using a const_iterator since we are not going to change the values.
//    MapDict::const_iterator end = _translations_.end();
//    for (MapDict::const_iterator it = _translations_.begin(); it != end; ++it)
//    {
//        std::cout <<  it->first << " : " << it->second << '\n';
//        std::cout <<  _translations_[it->first] << '\n';
//    }
    return 0;
}

int save_translations(){
    if(_translations_is_dirty){
        FILE *fp = fopen(translations_file_name, "wb");
        if(fp){
            // Iterate over the map and print out all key/value pairs.
            // Using a const_iterator since we are not going to change the values.
            MapDict::const_iterator end = _translations_.end();
            for (MapDict::const_iterator it = _translations_.begin(); it != end; ++it)
            {
                //std::cout <<  it->first << " : " << it->second << '\n';
                //std::cout <<  _translations_[it->first] << '\n';
                //fwrite(it->first.c_str(), 1, it->first.size(), fp);
                csv_fwrite(fp, it->first.c_str(), it->first.size());
                fwrite(",", 1, 1, fp);
                //fwrite(it->second.c_str(), 1, it->second.size(), fp);
                csv_fwrite(fp, it->second.c_str(), it->second.size());
                fwrite("\n", 1, sizeof("\n")-1, fp);
            }
            fclose(fp);
        }
    }
    return 0;
}

const char* _tr(const char* str)
{
    std::string tr_str = _translations_[str];
    if(tr_str.size()){
        return tr_str.c_str();
    }
    _translations_is_dirty = true;
    _translations_[str] = str;
    return str;
}
