#include "help-gui.h"
#include <FL/fl_ask.H>
#include "i18n_function.h"
#include "markdowncxx.h"
#include <stack>

const char *hlp_link(Fl_Widget *w, const char *uri);

class OurHelpWindow : public HelpWindow
{
public:
    typedef OurHelpWindow THISCLASS;
    const char *_help_file_name;
    std::stack<int> _navigation;
    int _last_found_pos;


    OurHelpWindow(): HelpWindow()
    {
        _help_file_name = "help.txt";
        _last_found_pos = 0;
        help_text->wrap_mode(1, 0);
        view_html->textsize(view_html->labelsize());
        view_html->link(hlp_link);
        THISMBACK(btnSavehelp, save_help_file);
        THISMBACK(btnSearchHelp, on_search);
        THISMBACK(words_to_search, on_search);
        THISMBACK(btnFontSmaller, on_font_size);
        THISMBACK(btnFontBigger, on_font_size);
        THISMBACK(btnBackward, on_navigate);
        THISMBACK(btnTop, on_navigate);
        THISMBACK(btnForward, on_navigate);
        THISMBACK(tabs, on_tab_change);
    }

    void refresh_html()
    {
        std::string html;
        markdown2html(help_text->buffer()->text(),
                      help_text->buffer()->length(), html);
        view_html->value(html.c_str());
        //printf(html.c_str());
    }

    void load_help_file(const char *fn=0)
    {
        fn = fn ? fn : _help_file_name;
        help_text->buffer()->loadfile(fn);
        _help_file_name = fn;
        refresh_html();
    }

    void save_help_file()
    {
        help_text->buffer()->savefile(_help_file_name);
        btnSavehelp->deactivate();
    }

    void on_search()
    {
        search_help_file(words_to_search->value());
    }

    void search_help_file(const char *topic)
    {
        words_to_search->value(topic);
        if(!topic) return;

        if(!help_text->buffer()->length())
        {
            load_help_file();
        }

        if(words_to_search->changed2())
        {
            _last_found_pos =0;
            words_to_search->clear_changed2();
        }

        int foundPos = 0;

        if(tabs->value() == tabView)
        {
            foundPos = view_html->find(topic, _last_found_pos);
            if(foundPos >= 0) _last_found_pos = foundPos;
            else foundPos = 0;
        }
        else
        {
            if(help_text->buffer()->search_forward(_last_found_pos, topic, &foundPos))
            {
                help_text->buffer()->select(foundPos, foundPos+strlen(topic));
                help_text->insert_position(foundPos);
                help_text->show_insert_position();
                _last_found_pos = foundPos+1;
                //delayed_focus(help_text);
            }
        }

        if(!foundPos)
        {
            _last_found_pos = 0;
            fl_alert(_tr("Nothing found for '%s' !"), topic);
        }
    }

    void on_font_size(Fl_Widget *wdg, void* udata)
    {
        Fl_Fontsize fsize = view_html->textsize();
        if(wdg == btnFontSmaller)
        {
            if (fsize > 10)
            {
                view_html->textsize(fsize - 2);
                view_html->reformat();
            }

            if (view_html->textsize() <= 10)
                btnFontSmaller->deactivate();
            btnFontBigger->activate();
        }
        else
        {
            if (fsize <= 24)
            {
                view_html->textsize(fsize + 2);
                view_html->reformat();
            }

            if (view_html->textsize() > 24)
                btnFontBigger->deactivate();
            btnFontSmaller->activate();
        }
    }
    void on_navigate(Fl_Widget *wdg, void* udata)
    {
        if(wdg == btnBackward && !_navigation.empty() )
        {
            view_html->topline(_navigation.top());
            _navigation.pop();
        }
        else if(wdg == btnTop)
        {
            view_html->topline(0);
        }
    }
    void on_tab_change(Fl_Widget *wdg, void* udata)
    {
        if(tabs->value() == tabView)
        {
            if(help_text->changed2())
            {
                refresh_html();
                help_text->clear_changed2();
                btnSavehelp->activate();
            }
        }
    }
};

const char *hlp_link(Fl_Widget *w, const char *uri)
{
    if(strncasecmp(uri, "db:/", 4) == 0)
    {
        printf("%s\n", uri);
        return 0;
    }
    hv::Fl_Help_View *hv = (hv::Fl_Help_View *)w;
    OurHelpWindow *hw = (OurHelpWindow *)w->window();
    hw->_navigation.push(hv->topline());
    return uri;
}

OurHelpWindow *app_help_window = 0;

int main(int argc, char *argv[])
{
    try
    {

        load_translations();
        //Fl::event_dispatch(&my_handle_events);
        //Fl_Input::default_number_format("\2,.");
        Fl::option(Fl::OPTION_ARROW_FOCUS, true);
        fl_register_images();
        Fl::scheme("gtk+");
        //set_app_font("erdana");
        Fl::visual(FL_RGB);

        //base_window_event_handler = app_window_handler;

        OurHelpWindow *win = new OurHelpWindow();
        //win->callback(&delete_on_window_close_cb);
#ifdef WIN32
        void *icon = (void *)LoadIcon(fl_display, "APP_ICON");
        if(icon) win->icon(icon);
#endif
        win->show(argc, argv);

        int result = Fl::run();

        save_translations();
        return result;
    }
    catch(...)
    {
        printf("catch(...)\n");
        fl_alert(_tr("Something unexpected happened, please ask your provider !"));
    }
    return -1;
}
